﻿using System;
using System.Text;
using System.IO;
using System.Xml;
using System.Data;
using System.Web;
using System.Collections;

namespace XML测试
{

    public class XMLdata
    {
        #region 私有成员
        /// <summary>
        /// 表名
        /// </summary>
        private string tableName = null;
        /// <summary>
        /// 零时数据集
        /// </summary>
        private DataSet tempDs = null;
        /// <summary>
        /// 删选条件 比如 "id='1' and userName='trace'" 
        /// </summary>
        private string where=null;
        /// <summary>
        /// 排序的字段 比如 "id desc,userName"
        /// </summary>
        private string sort = null;
        /// <summary>
        /// 表的字段名集合
        /// </summary>
        private string[] fields = null;
        /// <summary>
        /// 字段对应的值 集合
        /// </summary>
        private string[] datas = null;
        #endregion
        #region 公共属性
        /// <summary>
        /// 表名
        /// </summary>
        public string TableName
        {
            set { this.tableName = value; }
            get { return this.tableName; }

        }
        /// <summary>
        /// 零时数据集
        /// </summary>
        public DataSet TempDs
        {
            set { this.tempDs = value; }
            get { return tempDs; }
        }
        /// <summary>
        /// 删选条件 比如 "id='1' and userName='trace'" 
        /// </summary>
        public string Where
        {
            set { this.where = value; }
            get { return this.where; }
        }
        /// <summary>
        /// 排序的字段 比如 "id desc,userName"
        /// </summary>
        public string Sort
        {
            set { this.sort = value; }
            get { return this.sort; }
        }
        /// <summary>
        /// 表的字段名集合
        /// </summary>
        public string[] Fields
        {
            set { this.fields = value; }
        }
        /// <summary>
        /// 字段对应的值 集合
        /// </summary>
        public string[] Datas
        {
            set { this.datas = value; }
        }
        #endregion
        
        public XMLdata()
        {
            //
            // TODO: 提供将XML当作数据库处理的一些方法
            //
        }
        /// <summary>
        /// 取得XML文件的内容并填入DataSet
        /// </summary>
        private void Open()
        {
            try
            {
                this.tempDs = new DataSet();
                FileStream fin;
                string FilePath = HttpRuntime.AppDomainAppPath + this.tableName + ".xml";
                fin = new FileStream(FilePath, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
                this.tempDs.ReadXml(fin);
                fin.Close();
            }
            catch (Exception ex)
            {
 
            }
        }
        /// <summary>
        /// 将操作结果写入XML
        /// </summary>
        private void Save()
        {
            try
            {
                string FilePath = HttpRuntime.AppDomainAppPath + this.tableName + ".xml";
                this.tempDs.WriteXml(FilePath, XmlWriteMode.WriteSchema);
            }
            catch (Exception ex)
            {
                
            }
        }

        /// <summary>
        /// 查找XML 数据库
        /// </summary>
        /// <returns>Datatable</returns>
        public DataTable Result()
        {

            if (this.tempDs == null) this.Open();

            DataView myDv = new DataView(this.tempDs.Tables[0]);

            if (where != null) myDv.RowFilter = this.where;

            myDv.Sort = this.sort;

            return GetDataTable(myDv);
        }
        private DataTable GetDataTable(DataView obDataView)
        {
            if (null == obDataView)
            {
                throw new ArgumentNullException("DataView", "Invalid DataView object specified");
            }
            DataTable obNewDt = obDataView.Table.Clone();
            int idx = 0;
            string[] strColNames = new string[obNewDt.Columns.Count];
            foreach (DataColumn col in obNewDt.Columns)
            {
                strColNames[idx++] = col.ColumnName;
            }
            IEnumerator viewEnumerator = obDataView.GetEnumerator();
            while (viewEnumerator.MoveNext())
            {
                DataRowView drv = (DataRowView)viewEnumerator.Current;
                DataRow dr = obNewDt.NewRow();
                try
                {
                    foreach (string strName in strColNames)
                    {
                        dr[strName] = drv[strName];
                    }
                }
                catch (Exception ex)
                {
                    //Trace.WriteLine(ex.Message);
                }
                obNewDt.Rows.Add(dr);
            }
            return obNewDt;
        }

        /// <summary>
        /// 取得特定的行
        /// 限定条件获取某一行或多行记录
        /// </summary>
        /// <returns>各行数据</returns>
        public DataRow[] SelectRows()
        {
            if (this.tempDs == null) this.Open();
            DataRow[] myRows = tempDs.Tables[0].Select(this.where);
            return myRows;
        }
        /// <summary>
        /// 往XML当中插入一条数据
        /// </summary>
        /// <returns>操作是否成功</returns>
        public bool Insert()
        {
            if (this.tempDs == null) this.Open();

            try
            {
                DataRow newRow = tempDs.Tables[0].NewRow();

                for (int i = 0; i < this.fields.Length; i++)
                {
                    newRow[this.fields[i]] = this.datas[i];
                }

                tempDs.Tables[0].Rows.Add(newRow);
                this.Save();
                return true;
            }
            catch (Exception ee)
            {
                return false;
            }
        }

        /// <summary>
        /// 更新数据,这个时候要确保strFields 与 strData 两个数组的维数一致
        /// </summary>
        /// <returns>是否更新成功</returns>
        public bool Update()
        {
            if (this.tempDs == null) this.Open();

            try
            {
                DataRow[] editRow = tempDs.Tables[0].Select(this.where);

                for (int j = 0; j < editRow.Length; j++)
                {
                    for (int i = 0; i < this.where.Length; i++)
                    {
                        editRow[j][this.fields[i]] = this.datas[i];
                    }
                }
                this.Save();
                return true;
            }
            catch (Exception ee)
            {
                
                return false;
            }
        }

        /// <summary>
        /// 删除数据
        /// </summary>
        /// <returns>是否删除成功</returns>
        public bool Delete()
        {
            if (this.tempDs == null) this.Open();

            try
            {
                DataRow[] editRow = tempDs.Tables[0].Select(this.where);
                for (int i = 0; i < editRow.Length; i++)
                {
                    editRow[i].Delete();
                }
                this.Save();
                return true;
            }
            catch (Exception ee)
            {
               
                return false;
            }
        }
    }
}